#!/usr/bin/env python

# This application is released under the GNU General Public License 
# v3 (or, at your option, any later version). You can find the full 
# text of the license under http://www.gnu.org/licenses/gpl.txt. 
# By using, editing and/or distributing this software you agree to 
# the terms and conditions of this license. 
# Thank you for using free software!
#
#(c) QB89Dragon 2007/8 <hughescih@hotmail.com>
#(c) Whise 2008 <helderfraga@gmail.com>
#
# XDG Menu Functions

import gtk
import gconf
import optparse
import os
import urllib
import xml.dom.minidom
import string
import gobject
import xdg.Menu
import xdg.IconTheme
import xdg.Locale
from xdg.DesktopEntry import *
from Popup_Menu import add_menuitem, add_image_menuitem
import gc
import Globals
import gettext
import locale
try:
	import bookmarks
except:pass

try:
	import gio
	isgio = True
except:
	print 'gio not found'
	isgio = False

locale.setlocale (locale.LC_ALL, "")

#gettext.bindtextdomain("libgnomeui-2.0", '/usr/share/locale-langpack')

def _(s):
	gettext.textdomain("synaptic")
	gettext.install("synaptic", '/usr/share/locale-langpack')
	text = gettext.gettext(s)
	if text  != s:
		return gettext.gettext(s)
	gettext.textdomain("gnome-panel-2.0")
	gettext.install("gnome-panel-2.0", '/usr/share/locale-langpack')
	text = gettext.gettext(s)
	if text  != s:
		return gettext.gettext(s)
	gettext.textdomain("xdg-user-dirs")
	gettext.install("xdg-user-dirs", '/usr/share/locale')
	text = gettext.gettext(s)
	if text  != s:
		return gettext.gettext(s)
	gettext.textdomain("nautilus")
	gettext.install("nautilus", '/usr/share/locale-langpack')
	text = gettext.gettext(s)
	if text  != s:
		return gettext.gettext(s)
	gettext.textdomain("gnome-control-center-2.0")
	gettext.install("gnome-control-center-2.0", '/usr/share/locale-langpack')
	text = gettext.gettext(s)
	if text  != s:
		return gettext.gettext(s)

	return gettext.gettext(s)



class XDGMenu(gobject.GObject):

	__gsignals__ = {
        'changed': (gobject.SIGNAL_RUN_LAST, gobject.TYPE_NONE, ()),
        }
	def __init__(self):
		gobject.GObject.__init__ (self)
		if not os.path.isdir(Globals.AutoStartDirectory):
			os.system('mkdir %s' % Globals.AutoStartDirectory)
		self.AppsFile = None
		self.SetFile = None
		if Globals.OnlyShowFavs: 
			self.BaseMenu = '<Favorites>'
			self.CacheApplications = None
		else:
			self.BaseMenu = xdg.Menu.parse("applications")
			self.CacheApplications = self.BaseMenu
			self.CacheSettings = xdg.Menu.parse("settings.menu")
			self.AppsFile = self.CacheApplications.Filename
			self.SetFile = self.CacheSettings.Filename

			if isgio:
				self.parentfile = '/usr/share/applications'
				if os.path.isdir(self.parentfile):
					self.current_file1 = gio.File(self.parentfile).monitor_directory()
					self.current_file1.connect_after("changed", self.monitor_callback)

				self.parentfile1 = Globals.HomeDirectory + '/.local/share/applications'
				if os.path.isdir(self.parentfile1):
					self.current_file2 = gio.File(self.parentfile1).monitor_directory()
					self.current_file2.connect_after("changed", self.monitor_callback)
				
				self.current_file = gio.File(self.AppsFile).monitor_file()
				self.current_file.connect_after("changed", self.monitor_callback)
				self.monitor=gio.volume_monitor_get()
				self.monitor.connect("drive-changed",self.on_drive_changed)


		self.recent_manager = gtk.recent_manager_get_default()
		self.recent_manager.connect("changed", self.onRecentChanged)
	
		self.Menu = self.BaseMenu
		self.PrevMenu = []
		self.searchresults = 0

		##### This loads the icon cache

		self.IconCachePath = Globals.HomeDirectory + "/.menuiconcache.xml"
		self.gconf_client = gconf.client_get_default()
		self.IconNames = []
		self.Icons = []
		self.IconPix = {"":""}

		##### looksup icon theme in gconf, we could also use : gtk.settings_get_default().get_property("gtk-icon-theme-name")

		self.gtkicontheme = gtk.icon_theme_get_default()
		self.icontheme = self.gconf_client.get_string('/desktop/gnome/interface/icon_theme')
		self.old_icontheme = self.icontheme

		##### Check if icon theme is stored in cache#################
		if os.path.isfile(self.IconCachePath):

			XMLSettings = xml.dom.minidom.parse(self.IconCachePath)
			try:
				self.cacheicontheme = XMLSettings.getElementsByTagName("cache")[0].attributes["IconTheme"].value
			except:
				print "[XDG] Recoverable Error: Loading of icon cache has failed! This could be due to a corrupted file or read permissions."
				self.cacheicontheme = None
				os.system('rm ' +Globals.HomeDirectory + "/.menuiconcache.xml")
			if  self.cacheicontheme == self.icontheme:
				print 'icon theme in cache'
				XBase = XMLSettings.childNodes[0].getElementsByTagName("Icon")
				for node in XBase:
					iconname = node.attributes["IconNames"].value
					self.IconNames.append(iconname)
					icon = node.attributes["Icons"].value
					self.Icons.append(icon)
					if os.path.exists(icon):
						pix = gtk.gdk.pixbuf_new_from_file_at_size(icon, Globals.PG_iconsize, Globals.PG_iconsize)
					else:
						pix =	self.geticonfile(Globals.BrokenImage)
					self.IconPix[iconname] = pix
			else:
				print 'icon theme not in cache - restarting cache'
				os.system('rm ' +Globals.HomeDirectory + "/.menuiconcache.xml")

		try:
			self.webbookmarker = bookmarks.BookmarksMenu().getBookmarks()
		except:pass
		self.ConstructMenu()
		gc.collect()


	def SystemSuspend(self):
		"""Shutdown the system"""

		bus = dbus.Bus(dbus.Bus.TYPE_SESSION)
		obj = bus.get_object('org.gnome.PowerManager', '/org/gnome/PowerManager')
		self._gpm = dbus.Interface (obj, "org.gnome.PowerManager")
		try:
			self._gpm.Suspend()
		except dbus.DBusException:
			# this will trigger a method timeout exception.
			# As a workaround we swallow it silently
			pass



	def Restart(self,menu='all'):
		"""Issues a restart on the menu"""
	
		if Globals.OnlyShowFavs: 
			if menu == 'all' or menu == 'settings':
				menu = 'favorites'
		
		if menu == 'all':
			print 'menu restart Applications'
			self.BaseMenu = self.CacheApplications
			self.Menu = self.BaseMenu
			self.ConstructMenu()
			return True
		elif menu == 'settings':
			print 'menu restart Settings'
			self.BaseMenu = self.CacheSettings
			self.Menu = self.BaseMenu
			self.ConstructMenu()
			return True
		elif menu == 'places':
			print 'menu restart bookmarks'
			self.BaseMenu = "<Bookmarks>"
			self.Menu = self.BaseMenu
			self.ConstructMenu()
			return True
		elif menu == 'auxiliary':
			print 'menu restart auxiliary'
			self.BaseMenu = "<AuxiliaryFunctions>"
			self.Menu = self.BaseMenu
			self.ConstructMenu()
			return True
		elif menu == 'recent':
			print 'menu restart recent'
			self.BaseMenu = "<RecentItems>"
			self.Menu = self.BaseMenu
			self.ConstructMenu()
		elif menu == 'recentapps':
			print 'menu restart recent'
			self.BaseMenu = "<RecentApps>"
			self.Menu = self.BaseMenu
			self.ConstructMenu()
		elif menu == 'webbookmarks':
			print 'menu restart web bookmarks'
			self.BaseMenu = "<WebBookmarks>"
			self.Menu = self.BaseMenu
			self.ConstructMenu()
			return True
		elif menu == 'favorites':
			print 'menu restart favorites'
			self.BaseMenu = "<Favorites>"
			self.Menu = self.BaseMenu
			self.ConstructMenu()
			return True
		elif menu == 'shutdown':
			print 'menu restart shutdown'
			self.BaseMenu = "<Shutdown>"
			self.Menu = self.BaseMenu
			self.ConstructMenu()
			return True
		elif menu == 'previous':
			self.Menu = self.BaseMenu
			print 'menu restart previous'
			self.ConstructMenu()
			return True
		return False


	def addtomenu(self,names,icons,types,paths,execs):
		"""adds item to menu"""
		self.L_Names.append(names)
		self.L_Icons.append(self.geticonfile(icons))
		self.L_Icons_menu.append(icons)
		self.L_Types.append(types)
		self.L_Paths.append(paths)
		self.L_Execs.append(execs)


	def readMountFile(self, filename):
		"""Reads fstab file"""
		fstab = []
		f = open(filename, 'r')
		for line in f:
			if (not line.isspace() and not line.startswith('#') and not line.lower().startswith('none')) :
				fstabline = line.split()
				if fstabline[1] != 'none' and fstabline[1] != '/proc': fstab.append(fstabline[1])
	
		fstab.sort()
		return fstab


	def monitor_callback(self, monitor, Gfile, event, data=None):
		"""Callback for when directories change"""

		#self.current_file.disconnect_by_func(self.monitor_callback)
		self.CacheSettings = xdg.Menu.parse("settings.menu")
		self.CacheApplications = xdg.Menu.parse("applications")
		if str(self.BaseMenu) =="Applications":self.BaseMenu = self.CacheApplications
		self.emit('changed')
		gc.collect()

        def recentSortCriterium(self, item1, item2):
		"""Sort files"""
                return cmp(item2.get_modified(), item1.get_modified())

	
	def onRecentChanged(self, manager):
		"""Recent files have changed"""
		print 'recents changed'
		if self.BaseMenu == "<RecentItems>":
			self.emit('changed')

	def on_drive_changed (self,widget,drive):
		"""Bookmarks have changed"""
		print 'drives changed'
		if self.BaseMenu == "<Bookmarks>":
			self.emit('changed')

	def ConstructMenu(self):
		"""Construct the menu"""		
	
		self.L_Names = []
		self.L_Icons = []
		self.L_Icons_menu = []
		self.L_Types = []
		self.L_Paths = []
		self.L_Execs = []
		self.searchresults = 0

		
		#==============================================================
		#AUXILIARY FUNCTIONS MENU
		#==============================================================

		if self.Menu == "<AuxiliaryFunctions>":

			self.addtomenu(_("About Me"),"user-info",5,"","gnome-about-me")
			self.addtomenu(_("Appearance"),"gnome-settings-theme",5,"","gnome-appearance-properties")
			self.addtomenu(_("Menu editor"),"alacarte",6,"","alacarte")
			self.addtomenu(_("Screensaver"),"preferences-desktop-screensaver",5,"","gnome-screensaver-preferences")
			self.addtomenu(_("System Monitor"),"utilities-system-monitor",5,"","gnome-system-monitor")
			self.addtomenu(_("GnoMenu settings utility"),Globals.GraphicsDirectory + "logo.png",7,"",Globals.ProgramDirectory+"GnoMenu-Settings.py")

		#==============================================================
		#RECENT ITEMS MENU
		#==============================================================

		elif self.Menu == "<Favorites>":

			client = self.gconf_client.get_list(Globals.gconf_app_key + '/favorites',1)
			icontheme = gtk.icon_theme_get_default()
			client.sort(key=str.upper)
			for elem in client:
					
				self.L_Names.append(elem.split(':')[0])
				try:
					self.L_Icons.append(self.geticonfile(elem.split(':').pop()))
					self.L_Icons_menu.append(elem.split(':').pop())
				except:
					self.L_Icons.append(self.geticonfile(Globals.BrokenImage))
					self.L_Icons_menu.append(Globals.BrokenImage)
				self.L_Types.append(1)
				self.L_Paths.append("")
				if len(elem.split(':'))==4:
					self.L_Execs.append(elem.split(':')[1]+':'+elem.split(':')[2])
				else:
					self.L_Execs.append(elem.split(':')[1])
			if client == []:
				self.addtomenu(_('No favorites\nUse the mouse right button\nto add or remove favorites'),Globals.BrokenImage,9,"","")

		elif self.Menu == "<WebBookmarks>":

			try:
				self.webbookmarker = bookmarks.BookmarksMenu().getBookmarks()
				for item in self.webbookmarker:
					self.addtomenu(item[0],item[3],3,item[1],item[1])	
			except:pass

		elif self.Menu == "<Shutdown>":

			if os.path.exists('/usr/bin/gnome-power-cmd'):

				self.addtomenu(_('Shutdown'),'gtk-quit',1,"",'/usr/bin/gnome-power-cmd shutdown')
				self.addtomenu(_('Reboot'),'gnome-session-reboot',1,"","/usr/bin/gnome-power-cmd reboot")
				self.addtomenu(_('Suspend'),'gnome-session-suspend',1,"","/usr/bin/gnome-power-cmd suspend")	
				self.addtomenu(_('Hibernate'),'gnome-session-hibernate',1,"","/usr/bin/gnome-power-cmd hibernate")	
			else:
				self.addtomenu(_('Shutdown'),'gtk-quit',1,"",'gnome-session-save --shutdown-dialog')
				self.addtomenu(_('Reboot'),'gnome-session-reboot',1,"","gnome-session-save --shutdown-dialog")
				self.addtomenu(_('Suspend'),'gnome-session-suspend',1,"","gnome-session-save --shutdown-dialog")	
				self.addtomenu(_('Hibernate'),'gnome-session-hibernate',1,"","gnome-session-save --shutdown-dialog")	
			self.addtomenu(_('Logout'),'gnome-session-logout',1,"","gnome-session-save --logout")
			self.addtomenu(_('Lock Screen'),'system-lock-screen',1,"","gnome-screensaver-command --lock")	


		
		elif self.Menu == "<RecentItems>":

		        #self.recent_manager.set_limit(Globals.RI_numberofitems)
			self.recent = self.recent_manager.get_items()
			self.recent1 = []
			self.recent.sort(self.recentSortCriterium)
			self.recent1[0:Globals.RI_numberofitems] = self.recent[0:Globals.RI_numberofitems]
			for item in self.recent1:
				self.addtomenu(item.get_short_name(),item.get_icon(Globals.PG_iconsize),3,"",item.get_uri())
			
			self.addtomenu(_('Clear recent documents'),gtk.STOCK_CLEAR,10,"","")
			self.sorted_list = []

		elif self.Menu == "<RecentApps>":

		        #self.recent_manager.set_limit(Globals.RI_numberofitems)
			self.recent = self.recent_manager.get_items()
			self.recent1 = []
			self.recent.sort(self.recentSortCriterium)
			self.recent1[0:Globals.RI_numberofitems] = self.recent[0:Globals.RI_numberofitems]
			for item in self.recent1:
				app = item.last_application()
				#print str(item.get_application_info(app))
				if app not in self.L_Names:
					self.addtomenu(app,item.get_icon(Globals.PG_iconsize),3,"",item.get_uri())

			self.addtomenu(_('Clear recent documents'),gtk.STOCK_CLEAR,10,"","")

			self.sorted_list = []

		elif self.Menu == "<Bookmarks>":

			self.addtomenu(_("File System"),"drive-harddisk",3,"/","/")

			if isgio:
			
				self.drives = self.monitor.get_connected_drives()
				for drive in self.drives:
					
					if drive.has_media():
						self.mounts = drive.get_volumes()
						for mount in self.mounts:
							ico = mount.get_icon()
							self.L_Names.append(mount.get_name())
							self.L_Icons.append(self.geticonfile(self.getgicon(ico)))
							self.L_Icons_menu.append(self.getgicon(ico))
							self.L_Types.append(3)
							self.L_Paths.append("")
							try:
								self.L_Execs.append(str(mount.get_mount().get_root().get_uri()).replace("file://",""))
							except:
								self.L_Execs.append("")

			self.addtomenu(_('Home'),"user-home",3,"",Globals.HomeDirectory)
			self.addtomenu(_('Computer'),"computer",3,"",'computer:///')
			self.addtomenu(_('Network'),"network",3,"",'network:///')
			self.addtomenu(_('Trash'),'user-trash',3,"",'trash:///')

			if os.path.isfile(Globals.HomeDirectory + "/.gtk-bookmarks"):
				f = open(Globals.HomeDirectory + "/.gtk-bookmarks", "r")
				data = f.readlines(600)
				f.close()
				f = None
				for i in data:
					
					self.RInames = urllib.url2pathname(i)
					self.RInames = self.RInames.replace("file://","").replace("%U","").replace("%F","").replace("\n","")
					self.RInamess = self.RInames.split("/").pop().split(' ')
					if len(self.RInamess) ==2: 
								self.RInames = self.RInamess[1]
								self.RInamess = self.RInamess[0]
					elif len(self.RInamess) ==4: 
								self.RInames = self.RInamess[2] + ' ' + self.RInamess[3]
								self.RInamess = self.RInamess[0] + ' ' + self.RInamess[1]
					elif len(self.RInamess) ==6: 
								self.RInames = self.RInamess[3] + ' ' + self.RInamess[4]  + ' ' + self.RInamess[5]
								self.RInamess = self.RInamess[0] + ' ' + self.RInamess[1]  + ' ' + self.RInamess[2]
					if isgio:
						Gfile = gio.File(self.RInames)
						tuble = [Gfile, Gfile.query_info("standard::*"), []]
						ico = tuble[1].get_icon()
						self.addtomenu(self.RInamess,self.getgicon(ico),3,"",self.RInames)
					else:
						self.addtomenu(self.RInamess,"folder",3,"",self.RInames)
						
		
		#==============================================================
		#XDG PROGRAMS MENU
		#==============================================================

		elif isinstance(self.Menu, xdg.Menu.Menu):

			if str(self.BaseMenu) =="Applications":
				if Globals.Shownetandemail == 1:
					self.AddInternetButtons()
				for entry in self.Menu.getEntries():
					if isinstance(entry, xdg.Menu.Menu):		#Folder
						self.addtomenu(entry.getName(),entry.getIcon(),0,entry.getPath(True),"")
					elif isinstance(entry, xdg.Menu.MenuEntry):	#Application
						self.app = entry.DesktopEntry
						name = self.app.getName()
						if name not in self.L_Names:
							self.addtomenu(name,self.app.getIcon(),1,"",self.app.getExec())

			
				
				if str(self.Menu) =="Applications":
					self.AddPlacesButton()
					self.AddSystemButton()

			elif str(self.BaseMenu) =="Desktop" or str(self.BaseMenu) =="Preferences" or str(self.BaseMenu) =="Administration":
				for entry in self.Menu.getEntries():
					if isinstance(entry, xdg.Menu.Menu):		#Folder
						self.addtomenu(entry.getName(),entry.getIcon(),0,entry.getPath(True),"")
					elif isinstance(entry, xdg.Menu.MenuEntry):	#Application
						self.app = entry.DesktopEntry
						name = self.app.getName()
						if name not in self.L_Names:
							self.addtomenu(name,self.app.getIcon(),1,"",self.app.getExec())
	
	


		else:
		#==============================================================
		#XDG PROGRAMS MENU (SEARCH)
		#==============================================================
			#self.searchresults = 0
			for entry in self.CacheApplications.getEntries():
				
				if isinstance(entry, xdg.Menu.Menu):		#Folder
					self.sname = str(entry.getName())
					self.sname = self.sname.upper()
				
					if self.sname.count(self.Menu) != 0:
						self.addtomenu(entry.getName(),entry.getIcon(),0,entry.getPath(True),"")
						self.searchresults=self.searchresults+1
					self.SearchMenu(self.CacheApplications.getMenu(entry.getPath(True)))
				elif isinstance(entry, xdg.Menu.MenuEntry):	#Application
					self.app = entry.DesktopEntry
					self.sname = str(self.app.getName())
					self.sname = self.sname.upper()
					if self.sname.count(self.Menu) != 0:
						self.addtomenu(self.app.getName(),self.app.getIcon(),1,"",self.app.getExec())
						self.searchresults=self.searchresults+1
			for entry in self.CacheSettings.getEntries():
				
				if isinstance(entry, xdg.Menu.Menu):		#Folder
					self.sname = str(entry.getName())
					self.sname = self.sname.upper()
				
					if self.sname.count(self.Menu) != 0:
						self.addtomenu(entry.getName(),entry.getIcon(),0,entry.getPath(True),"")
						self.searchresults=self.searchresults+1
					self.SearchMenu(self.CacheSettings.getMenu(entry.getPath(True)))
				elif isinstance(entry, xdg.Menu.MenuEntry):	#Application
					self.app = entry.DesktopEntry
					self.sname = str(self.app.getName())
					self.sname = self.sname.upper()
					if self.sname.count(self.Menu) != 0:
						self.addtomenu(self.app.getName(),self.app.getIcon(),1,"",self.app.getExec())
						self.searchresults=self.searchresults+1
			if self.searchresults==0:
				self.addtomenu("<separator>","",8,"","")
				self.addtomenu(_("No items matched your search criteria"),"",9,"","")
			else:
				self.addtomenu
				self.addtomenu("<separator>","",8,"","")
				self.addtomenu(_("Found %s results to your search") % str(self.searchresults),"",9,"","")
				
		#Add back button so long as it isn't the base XDG menu
		if isinstance(self.Menu, xdg.Menu.Menu):
			
			if str(self.Menu) !="Applications":
				self.AddBackButton()
		else:
			if Globals.MenuTabCount == 0:
				if not Globals.OnlyShowFavs:
					self.AddBackButton()
				else:
					if self.Menu != "<Favorites>":
						self.AddBackButton()
		gc.collect()

	def AddPlacesButton(self):
		"""Add Places buttons"""

		a = "Places"
		if _("Places") != None:
			a = _("Places")
		self.addtomenu(a,"drive-harddisk",2,"","")


	def AddSystemButton(self):
		"""Add System buttons"""

		a = "System"
		if _("System") != None:
			a = _("System")
		self.addtomenu(a,"stock_properties",2,"","")


	def SearchMenu(self, folder):
		for entry in folder.getEntries():
			
			if isinstance(entry, xdg.Menu.Menu):		#Folder
				self.sname = str(entry.getName())
				self.sname = self.sname.upper()
				if self.sname.count(self.Menu) != 0:
					self.addtomenu(entry.getName(),entry.getIcon(),0,entry.getPath(True),"")
					self.searchresults=self.searchresults+1
				#self.SearchMenu(self.BaseMenu.getMenu(entry.getPath(True)))
			elif isinstance(entry, xdg.Menu.MenuEntry):	#Application
				self.app = entry.DesktopEntry
				self.sname = str(self.app.getName())
				self.sname = self.sname.upper()
				if self.sname.count(self.Menu) != 0:
					self.addtomenu(self.app.getName(),self.app.getIcon(),1,"",self.app.getExec())
					self.searchresults=self.searchresults+1

	def AddInternetButtons(self):
		"""Add Internet and Email buttons"""

		gconf_client = gconf.client_get_default()
		self.client = gconf_client.get_string("/desktop/gnome/applications/browser/exec")
		# FIXME Languages
		a = "Browse Internet"
		if _("Browse Internet") != None:
			a = _("Browse Internet")
		self.addtomenu(a,"web-browser",1,"",self.client)
		
		self.client = gconf_client.get_string("/desktop/gnome/url-handlers/mailto/command")
		a = "E-mail"
		if _("E-mail") != None:
			a = _("E-mail")
		self.addtomenu(a,"evolution",1,"",self.client)
		#self.addtomenu("<separator>","",8,"","")
		
	def AddBackButton(self):
		"""Add Back buttons"""

		self.addtomenu(_("Back"),"stock_left",2,"","")
		

	def ButtonClick(self,index,event=0):
		"""Callback when buttons are clicked"""

		self.etype = self.L_Types[index]
		if event.type == gtk.gdk.KEY_PRESS:event_button = 1
		elif event.type == gtk.gdk.BUTTON_PRESS:event_button = event.button
		if event_button == 1:
		
			places = "Places"
			if _("Places") != None:
				places = _("Places")
			system = "System"
			if _("System") != None:
				system = _("System")
	
			if self.etype == 0:     				#Subfolder
				self.PrevMenu.append(self.Menu)
				self.Menu = self.BaseMenu.getMenu(self.L_Paths[index])
				self.ConstructMenu()
			elif self.etype == 1:                   #Application
				print "launching item from application list..."
				self.expl = self.L_Execs[index].split()
				#if last item in list begins with the character "%" then remove it and add it after the ' mark
				self.outcommand = ""
				if "%" in self.expl[len(self.expl)-1]:
					self.outcommand = " " +  self.expl.pop()
				self.expls = string.join(self.expl,' ')			
				if self.expls == 'trash:///':
					self.expls = "xdg-open trash:///"
				elif self.expls == 'network:///':
					self.expls = "xdg-open network:///"
				else:
					if os.path.isdir(self.expls):
						self.expls = "xdg-open '" + self.expls +"'"+self.outcommand
					else:
						self.expls = "/bin/sh -c '" + self.expls +"'"+self.outcommand
				os.system(self.expls+' &')
				self.Restart('previous')
				return 1
			elif self.etype == 2:                              #Menu Back button
				if self.L_Names[index] == _('Back'):
					try:
						self.Menu = self.PrevMenu.pop()
					except:
						self.BaseMenu = self.CacheApplications
						self.Menu=self.BaseMenu
					if self.Menu == 'Applications':
						self.BaseMenu = self.CacheApplications
						self.Menu=self.BaseMenu
					self.ConstructMenu()
				elif self.L_Names[index] == system:
					self.PrevMenu.append(self.Menu)
					self.Restart('settings')
				elif self.L_Names[index] == places:
					self.PrevMenu.append(self.Menu)
					self.Restart('places')
			
			elif self.etype == 3:                                   #Recent Items button (file)
				os.system("xdg-open '"+self.L_Execs[index]+"' &")
				self.Restart('previous')
			elif self.etype == 4:                                                   #Recent Items button (directory)
				os.system("xdg-open '"+self.L_Execs[index]+"' &")
				self.Restart('previous')
			elif self.etype == 5:                                                       #Launch straight file from execlist (NoWait)
				os.system("/bin/sh -c '" + self.L_Execs[index]+"' &")
			elif self.etype == 6:                                                   #Items in the menu have changed
				os.system("/bin/sh -c '" + self.L_Execs[index]+"' &")
			elif self.etype == 7:                                                   #Properties of the menu have changed
				os.system("/bin/sh -c '" + self.L_Execs[index]+"' &")
				Globals.ReloadSettings()
			elif self.etype == 10: #Clear Recent 
				self.recent_manager.purge_items()
				self.Restart('previous')

			#elif self.etype == 9:     				#Subfolder
			#	self.PrevMenu.append(self.Menu)
			#	self.BaseMenu = xdg.Menu.parse("settings.menu")
			#	self.Menu=self.BaseMenu
			#	self.ConstructMenu()
		elif event_button == 3:
			self.m = gtk.Menu()
			#print self.L_Names[index]+ ':' + self.L_Execs[index] + ':' + self.Iconnames[index]
		
			favlist = self.gconf_client.get_list(Globals.gconf_app_key + '/favorites',1)
			add_image_menuitem(self.m, self.L_Icons_menu[index], self.L_Names[index], self.dummy,self.L_Names[index],self.L_Execs[index],self.L_Icons_menu[index],self.L_Execs[index])
			add_menuitem(self.m, "-")
			if self.L_Names[index]+ ':' + self.L_Execs[index] + ':' + self.L_Icons_menu[index] not in favlist and self.etype != 0:
				add_image_menuitem(self.m, gtk.STOCK_ADD, _("Add to Favorites"), self.addfav,self.L_Names[index],self.L_Execs[index],self.L_Icons_menu[index],self.L_Execs[index])
			if self.L_Names[index]+ ':' + self.L_Execs[index] + ':' + self.L_Icons_menu[index] in favlist:
				add_image_menuitem(self.m, gtk.STOCK_REMOVE, _("Remove from Favorites"), self.removefav,self.L_Names[index],self.L_Execs[index],self.L_Icons_menu[index],self.L_Execs[index])
			add_menuitem(self.m, "-")
			if  self.etype != 0:
				add_image_menuitem(self.m, gtk.STOCK_HOME, _("Create Desktop Shortcut"), self.addshort,self.L_Names[index],self.L_Execs[index],self.L_Icons_menu[index],self.L_Execs[index])
				add_menuitem(self.m, "-")
				if (str(self.L_Names[index])+'.desktop') in os.listdir(Globals.AutoStartDirectory):
					add_image_menuitem(self.m, gtk.STOCK_REMOVE, _("Remove from System Startup"), self.remove_autostarter,self.L_Names[index],self.L_Execs[index],self.L_Icons_menu[index],self.L_Execs[index])
				else:
					add_image_menuitem(self.m, gtk.STOCK_ADD, _("Add to System Startup"), self.create_autostarter,self.L_Names[index],self.L_Execs[index],self.L_Icons_menu[index],self.L_Execs[index])	
			self.m.show_all()
			self.m.popup(None, None, None, event.button, event.time)
			

	def dummy(self,widget,name,execs,ico):
		pass

	def addfav(self,widget,name,execs,ico):
		"""Add to Favorites"""

		favlist = self.gconf_client.get_list(Globals.gconf_app_key + '/favorites',1)
		if name+ ':' + execs + ':' + ico not in favlist:
			favlist.append(name+ ':' + execs + ':' + ico)
			self.gconf_client.set_list(Globals.gconf_app_key + '/favorites',1,favlist)
		
	def removefav(self,widget,name,execs,ico):
		"""Remove from Favorites"""

		favlist = self.gconf_client.get_list(Globals.gconf_app_key + '/favorites',1)
		if name+ ':' + execs + ':' + ico in favlist:
			favlist.remove(name+ ':' + execs+ ':' + ico)
			self.gconf_client.set_list(Globals.gconf_app_key + '/favorites',1,favlist)
			self.emit('changed')

	def create_autostarter (self,widget,name,execs,ico):
		"""Create Autostarter"""

		if not os.path.isdir(Globals.AutoStartDirectory):
			os.system('mkdir %s' % Globals.AutoStartDirectory)
		if (str(name)+'.desktop') not in os.listdir(Globals.AutoStartDirectory):
			self.addshort(widget,name,execs,ico,Globals.AutoStartDirectory)

	def remove_autostarter (self,widget,name,execs,ico):
		"""Remove Autostarter"""

		if not os.path.isdir(Globals.AutoStartDirectory):
			os.system('mkdir %s' % Globals.AutoStartDirectory)
		if (str(name)+'.desktop')  in os.listdir(Globals.AutoStartDirectory):
			os.system('rm %s%s' % (chr(34)+Globals.AutoStartDirectory,(str(name)+'.desktop')+chr(34)))
			
					
	def addshort(self,widget,name,execs,ico,desk='desktop'):
		"""Add Desktop shortcut"""

		if desk =='desktop':desk = self.xdg_dir("XDG_DESKTOP_DIR")
		starter = '%s/%s.desktop' % (desk, name)
		code = ['#!/usr/bin/env xdg-open','[Desktop Entry]']
		code.append('Name=%s' % name)
		code.append('StartupNotify=true')
		code.append('Terminal=false')
		code.append('Version=1.0')
		code.append('Icon=%s' % ico)
		code.append('Type=Application')
		code.append('Exec= %s' % execs)
		code.append('X-GNOME-Autostart-enabled=true')
		#print code
		f = open(starter, 'w')
		if f:
			for l in code:
				f.write(l + '\n')
			f.close()
			
		
	def xdg_dir(self,key):
		"""http://www.freedesktop.org/wiki/Software/xdg-user-dirs"""

		user_dirs_dirs = os.path.expanduser("~/.config/user-dirs.dirs")
		if os.path.exists(user_dirs_dirs):
			f = open(user_dirs_dirs, "r")
			for line in f.readlines():
				if line.startswith(key):
					return os.path.expandvars(line[len(key)+2:-2])
		return False


	def Icon_change(self):
		"""Icons have changed"""

		self.icontheme = self.gconf_client.get_string('/desktop/gnome/interface/icon_theme')
		self.gtkicontheme = gtk.icon_theme_get_default()
		self.IconPix.clear()



	def getgicon(self,gico):
		"""Returns gio icon"""

		names = self.gtkicontheme.lookup_by_gicon(gico,Globals.PG_iconsize,gtk.ICON_LOOKUP_USE_BUILTIN);
		if names:
			return names.get_filename()
		return 'gtk-execute'
		#names = gico.get_names()
		#for x in range(0,len(names)):
		#	if self.gtkicontheme.has_icon(names[x]):
		#		return names[x]
		#return 'gtk-execute'

	def addicontomen(self,icon,iconpath,pix):
		"""Adds icons to cache"""
		self.IconNames.append(icon)
		self.Icons.append(iconpath)
		self.IconPix[icon] = pix
		gc.collect()
		

	def geticonfile(self,icon):
		"""Returns system icon paths"""
	
		if type(icon).__name__ == 'Pixbuf':
			return icon

		if self.icontheme != self.old_icontheme:
				
			self.IconNames = []
			self.Icons = []
			self.IconPix.clear()
			self.old_icontheme = self.icontheme	
		# This code caches icons after the first load so they don't need to get reloaded IS THIS FASTER??????
		if icon in self.IconNames:

			return self.IconPix[icon]#gtk.gdk.pixbuf_new_from_file_at_size(icon, Globals.PG_iconsize, Globals.PG_iconsize)
		else:
			self.iconinfo = self.gtkicontheme.lookup_icon(str(icon),Globals.PG_iconsize,gtk.ICON_LOOKUP_USE_BUILTIN);
			if self.iconinfo:
				iconpath = self.iconinfo.get_filename()
				pix = gtk.gdk.pixbuf_new_from_file_at_size(iconpath, Globals.PG_iconsize, Globals.PG_iconsize)
				self.addicontomen(icon,iconpath,pix)
				return pix
			else:
				iconpath = xdg.IconTheme.getIconPath(icon, Globals.PG_iconsize, self.icontheme, ["png", "xpm", "svg"])
				if not iconpath:
					iconpath = "/usr/share/icons/application-default-icon.png"
			pix = gtk.gdk.pixbuf_new_from_file_at_size(iconpath, Globals.PG_iconsize, Globals.PG_iconsize)
			self.addicontomen(icon,iconpath,pix)
			return pix


	def CallSpecialMenu(self,command,data=None):
		"""Call was placed on the menu"""

		command = int(command)
		a = str(self.Menu)
		if command==0:	#Back button
			if self.PrevMenu:
				self.Menu = self.PrevMenu.pop()
				self.ConstructMenu()
		elif command==1:	#Go to base menu and reload it
			self.PrevMenu = []
			self.Restart()
		elif command==2:	#Go to unmount drives menu
			if a != '<RecentApps>':
				self.PrevMenu.append(self.Menu)
				self.Restart('recentapps')
		elif command==3:	#Auxiliary functions menu
			if a != '<AuxiliaryFunctions>':
				self.PrevMenu.append(self.Menu)
				self.Restart('auxiliary')
		elif command==4:	#Recent Items Menu
			if a != '<RecentItems>':
				self.PrevMenu.append(self.Menu)
				self.Restart('recent')
		elif command==5:	#Search
			if data!="":
				self.PrevMenu = [self.BaseMenu]
				self.Menu=data.upper()
				self.ConstructMenu()
			else:
				self.PrevMenu = []
				self.Menu=self.BaseMenu
				self.ConstructMenu()
		elif command==6:	#Launch first item in search list
			if self.searchresults!=0:
				if Globals.Shownetandemail == 1:
					self.ButtonClick(3)
				else:
					self.ButtonClick(0)
		elif command==7:	#Recent Items Menu
			if a != '<Favorites>':
				self.PrevMenu.append(self.Menu)
				self.Restart('favorites')
		elif command==8:	#Recent Items Menu
			if a != '<Bookmarks>':
				self.PrevMenu.append(self.Menu)
				self.Restart('places')
		elif command==9:	#Recent Items Menu
			if a != '<Shutdown>':
				self.PrevMenu.append(self.Menu)
				self.Restart('shutdown')
		elif command==10:	#Recent Items Menu
			if a != '<WebBookmarks>':
				self.PrevMenu.append(self.Menu)
				self.Restart('webbookmarks')
	
	def destroy(self):
		print "Saving Icon Cache..."
		#Save Imagecache
		XMLSettings = xml.dom.minidom.Document()
		XBase = XMLSettings.createElement("cache")
		XBase.setAttribute("IconTheme", self.icontheme)
		XMLSettings.appendChild(XBase)
		for icon in range(0,len(self.IconNames)):
			node = XMLSettings.createElement("Icon")
			node.setAttribute("IconNames", self.IconNames[icon])
			node.setAttribute("Icons", self.Icons[icon])
			XBase.appendChild(node)
		#Now write it to the file
		file = open(self.IconCachePath,"w")
		XMLSettings.writexml(file, "    ", "", "\n", "UTF-8")
		XMLSettings.unlink()
		print "XDG: Icon cache successfully written to user's home directory"		
